#include "glow/IR/IR.h"
#include "glow/IR/Instrs.h"
#include "glow/Base/Type.h"
#include "glow/Support/Support.h"

using namespace glow;

void AllocActivationInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Ty: " << getTy()
     << "}";
}

void TensorViewInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Ty: " << getTy()
     << ", Offsets: " << getOffsets()
     << "}";
}

void DeallocActivationInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}
  AllocActivationInst *DeallocActivationInst::getAlloc() const { return  llvm::cast<AllocActivationInst>(getSrc()); }

void CopyInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void ConvolutionGradInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Kernels: " << getKernels()
     << ", Strides: " << getStrides()
     << ", Pads: " << getPads()
     << ", Group: " << getGroup()
     << ", Dilation: " << getDilation()
     << ", Layout: " << getLayout()
     << ", FusedActivation: " << getFusedActivation()
     << "}";
}

void ConvolutionInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Kernels: " << getKernels()
     << ", Strides: " << getStrides()
     << ", Pads: " << getPads()
     << ", Group: " << getGroup()
     << ", Dilation: " << getDilation()
     << ", Layout: " << getLayout()
     << ", FusedActivation: " << getFusedActivation()
     << "}";
}

void ChannelwiseQuantizedConvolutionInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Kernels: " << getKernels()
     << ", Strides: " << getStrides()
     << ", Pads: " << getPads()
     << ", Group: " << getGroup()
     << ", Groupwise: " << getGroupwise()
     << "}";
}

void Convolution3DGradInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Kernels: " << getKernels()
     << ", Strides: " << getStrides()
     << ", Pads: " << getPads()
     << ", Group: " << getGroup()
     << "}";
}

void Convolution3DInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Kernels: " << getKernels()
     << ", Strides: " << getStrides()
     << ", Pads: " << getPads()
     << ", Group: " << getGroup()
     << "}";
}

void MaxPoolWithArgmaxGradInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Kernels: " << getKernels()
     << ", Strides: " << getStrides()
     << ", Pads: " << getPads()
     << ", Layout: " << getLayout()
     << "}";
}

void MaxPoolWithArgmaxInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Kernels: " << getKernels()
     << ", Strides: " << getStrides()
     << ", Pads: " << getPads()
     << ", Layout: " << getLayout()
     << "}";
}

void MaxPoolInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Kernels: " << getKernels()
     << ", Strides: " << getStrides()
     << ", Pads: " << getPads()
     << ", Layout: " << getLayout()
     << "}";
}

void AvgPoolGradInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Kernels: " << getKernels()
     << ", Strides: " << getStrides()
     << ", Pads: " << getPads()
     << ", Layout: " << getLayout()
     << "}";
}

void AvgPoolInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Kernels: " << getKernels()
     << ", Strides: " << getStrides()
     << ", Pads: " << getPads()
     << ", Layout: " << getLayout()
     << "}";
}

void ArgMaxInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Axis: " << getAxis()
     << ", KeepDims: " << getKeepDims()
     << "}";
}

void AdaptiveAvgPoolGradInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void AdaptiveAvgPoolInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void FullyConnectedInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void RowwiseQuantizedFullyConnectedInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void LocalResponseNormalizationGradInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " HalfWindowSize: " << getHalfWindowSize()
     << ", Alpha: " << getAlpha()
     << ", Beta: " << getBeta()
     << ", K: " << getK()
     << "}";
}

void LocalResponseNormalizationInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " HalfWindowSize: " << getHalfWindowSize()
     << ", Alpha: " << getAlpha()
     << ", Beta: " << getBeta()
     << ", K: " << getK()
     << "}";
}

void SoftMaxInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void SoftMaxGradInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void CrossEntropyLossInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void CrossEntropyLossGradInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void MatMulInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void BatchMatMulInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void BatchedReduceAddInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Axis: " << getAxis()
     << "}";
}

void BatchedReduceMinInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Axes: " << getAxes()
     << "}";
}

void LengthsSumInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void SparseLengthsSumGradInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void SparseLengthsSumInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void SparseLengthsWeightedSumGradInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void SparseLengthsWeightedSumInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void EmbeddingBagInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " HasEndOffset: " << getHasEndOffset()
     << "}";
}

void RowwiseQuantizedSparseLengthsWeightedSumInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " UseFP16Accumulation: " << getUseFP16Accumulation()
     << "}";
}

void FusedRowwiseQuantizedSparseLengthsWeightedSumInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " UseFP16Accumulation: " << getUseFP16Accumulation()
     << "}";
}

void EmbeddingBagByteRowwiseOffsetsInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " UseFP16Accumulation: " << getUseFP16Accumulation()
     << ", HasEndOffset: " << getHasEndOffset()
     << "}";
}

void LengthsToRangesInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void LengthsRangeFillInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void SparseToDenseInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void SparseToDenseMaskInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Mask: " << getMask()
     << "}";
}

void BatchedAddInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void ElementAddInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void ElementSubInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void ElementMulInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void ElementDivInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void ElementMaxInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void ElementMinInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void ElementCmpLTEInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void ElementCmpEQInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void ElementCmpLTInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void ElementIsNaNInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void ElementPowInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void ElementLogInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void ElementExpInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void ElementSelectInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void ModuloInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Divisor: " << getDivisor()
     << ", SignFollowDivisor: " << getSignFollowDivisor()
     << "}";
}

void ReluGradInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void ReluInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void SigmoidInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void TanhInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void TransposeInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Shuffle: " << getShuffle()
     << "}";
}

void SplatInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Value: " << getValue()
     << "}";
}

void TouchInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void InsertTensorInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Offsets: " << getOffsets()
     << ", Count: " << getCount()
     << ", Axis: " << getAxis()
     << "}";
}

void ExtractTensorInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Offsets: " << getOffsets()
     << "}";
}

void GatherInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " BatchDims: " << getBatchDims()
     << "}";
}

void GatherRangesInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void ScatterDataInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Cumulative: " << getCumulative()
     << "}";
}

void BatchOneHotInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void SpaceToDepthInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " BlockSize: " << getBlockSize()
     << "}";
}

void ResizeNearestInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " HeightScale: " << getHeightScale()
     << ", WidthScale: " << getWidthScale()
     << "}";
}

void FlipInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Axis: " << getAxis()
     << "}";
}

void DebugPrintInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void TraceEventInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Index: " << getIndex()
     << "}";
}

void QuantizationProfileInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void IntLookupTableInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void QuantizeInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void DequantizeInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void RescaleQuantizedInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void TopKInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " K: " << getK()
     << "}";
}

void ConvertToInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
}

void CPUMaxSplatInst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " SplatValue: " << getSplatValue()
     << "}";
}

void CPUConvDKKC8Inst::dump(llvm::raw_ostream &os) const {
  os << "%" << (std::string) getName() << " = " << getKindName() << " ";
  dumpOperands(os);
  os << " {"
     << " Kernels: " << getKernels()
     << ", Strides: " << getStrides()
     << ", Pads: " << getPads()
     << ", Group: " << getGroup()
     << "}";
}

#include "glow/CPUSpecificInstrsVerification.h"
